/* This file was generated by upb_generator from the input file:
 *
 *     envoy/service/discovery/v3/ads.proto
 *
 * Do not edit -- your changes will be discarded when the file is
 * regenerated.
 * NO CHECKED-IN PROTOBUF GENCODE */

#include <stddef.h>
#include "upb/generated_code_support.h"
#include "envoy/service/discovery/v3/ads.upb_minitable.h"
#include "envoy/service/discovery/v3/discovery.upb_minitable.h"
#include "udpa/annotations/status.upb_minitable.h"
#include "udpa/annotations/versioning.upb_minitable.h"

// Must be last.
#include "upb/port/def.inc"

extern const struct upb_MiniTable UPB_PRIVATE(_kUpb_MiniTable_StaticallyTreeShaken);
const upb_MiniTable envoy__service__discovery__v3__AdsDummy_msg_init = {
  NULL,
  NULL,
  8, 0, kUpb_ExtMode_NonExtendable, 0, UPB_FASTTABLE_MASK(255), 0,
#ifdef UPB_TRACING_ENABLED
  "envoy.service.discovery.v3.AdsDummy",
#endif
};

const upb_MiniTable* envoy__service__discovery__v3__AdsDummy_msg_init_ptr = &envoy__service__discovery__v3__AdsDummy_msg_init;
static const upb_MiniTable *messages_layout[1] = {
  &envoy__service__discovery__v3__AdsDummy_msg_init,
};

const upb_MiniTableFile envoy_service_discovery_v3_ads_proto_upb_file_layout = {
  messages_layout,
  NULL,
  NULL,
  1,
  0,
  0,
};

#include "upb/port/undef.inc"

