/* This file was generated by upb_generator from the input file:
 *
 *     envoy/config/trace/v3/trace.proto
 *
 * Do not edit -- your changes will be discarded when the file is
 * regenerated.
 * NO CHECKED-IN PROTOBUF GENCODE */

#ifndef ENVOY_CONFIG_TRACE_V3_TRACE_PROTO_UPB_H__UPB_H_
#define ENVOY_CONFIG_TRACE_V3_TRACE_PROTO_UPB_H__UPB_H_

#include "upb/generated_code_support.h"

/* Public Imports. */
#include "envoy/config/trace/v3/datadog.upb.h"
#include "envoy/config/trace/v3/dynamic_ot.upb.h"
#include "envoy/config/trace/v3/http_tracer.upb.h"
#include "envoy/config/trace/v3/lightstep.upb.h"
#include "envoy/config/trace/v3/opentelemetry.upb.h"
#include "envoy/config/trace/v3/service.upb.h"
#include "envoy/config/trace/v3/zipkin.upb.h"

#include "envoy/config/trace/v3/trace.upb_minitable.h"

#include "envoy/config/trace/v3/datadog.upb_minitable.h"
#include "envoy/config/trace/v3/dynamic_ot.upb_minitable.h"
#include "envoy/config/trace/v3/http_tracer.upb_minitable.h"
#include "envoy/config/trace/v3/lightstep.upb_minitable.h"
#include "envoy/config/trace/v3/opentelemetry.upb_minitable.h"
#include "envoy/config/trace/v3/service.upb_minitable.h"
#include "envoy/config/trace/v3/zipkin.upb_minitable.h"

// Must be last.
#include "upb/port/def.inc"

#ifdef __cplusplus
extern "C" {
#endif



#ifdef __cplusplus
}  /* extern "C" */
#endif

#include "upb/port/undef.inc"

#endif  /* ENVOY_CONFIG_TRACE_V3_TRACE_PROTO_UPB_H__UPB_H_ */
