/* This file was generated by upb_generator from the input file:
 *
 *     envoy/type/v3/ratelimit_unit.proto
 *
 * Do not edit -- your changes will be discarded when the file is
 * regenerated.
 * NO CHECKED-IN PROTOBUF GENCODE */



#ifndef ENVOY_TYPE_V3_RATELIMIT_UNIT_PROTO_UPB_H__UPBDEFS_H_
#define ENVOY_TYPE_V3_RATELIMIT_UNIT_PROTO_UPB_H__UPBDEFS_H_

#include "upb/reflection/def.h"
#include "upb/reflection/internal/def_pool.h"

#include "upb/port/def.inc"

#ifdef __cplusplus
extern "C" {
#endif

extern _upb_DefPool_Init envoy_type_v3_ratelimit_unit_proto_upbdefinit;


#ifdef __cplusplus
}  /* extern "C" */
#endif

#include "upb/port/undef.inc"

#endif  /* ENVOY_TYPE_V3_RATELIMIT_UNIT_PROTO_UPB_H__UPBDEFS_H_ */
