/* This file was generated by upb_generator from the input file:
 *
 *     envoy/extensions/filters/network/http_connection_manager/v3/http_connection_manager.proto
 *
 * Do not edit -- your changes will be discarded when the file is
 * regenerated.
 * NO CHECKED-IN PROTOBUF GENCODE */

#ifndef ENVOY_EXTENSIONS_FILTERS_NETWORK_HTTP_CONNECTION_MANAGER_V3_HTTP_CONNECTION_MANAGER_PROTO_UPB_H__UPB_MINITABLE_H_
#define ENVOY_EXTENSIONS_FILTERS_NETWORK_HTTP_CONNECTION_MANAGER_V3_HTTP_CONNECTION_MANAGER_PROTO_UPB_H__UPB_MINITABLE_H_

#include "upb/generated_code_support.h"

// Must be last.
#include "upb/port/def.inc"

#ifdef __cplusplus
extern "C" {
#endif

extern const upb_MiniTable envoy__extensions__filters__network__http_0connection_0manager__v3__HttpConnectionManager_msg_init;
extern const upb_MiniTable* envoy__extensions__filters__network__http_0connection_0manager__v3__HttpConnectionManager_msg_init_ptr;
extern const upb_MiniTable envoy__extensions__filters__network__http_0connection_0manager__v3__HttpConnectionManager__Tracing_msg_init;
extern const upb_MiniTable* envoy__extensions__filters__network__http_0connection_0manager__v3__HttpConnectionManager__Tracing_msg_init_ptr;
extern const upb_MiniTable envoy__extensions__filters__network__http_0connection_0manager__v3__HttpConnectionManager__InternalAddressConfig_msg_init;
extern const upb_MiniTable* envoy__extensions__filters__network__http_0connection_0manager__v3__HttpConnectionManager__InternalAddressConfig_msg_init_ptr;
extern const upb_MiniTable envoy__extensions__filters__network__http_0connection_0manager__v3__HttpConnectionManager__SetCurrentClientCertDetails_msg_init;
extern const upb_MiniTable* envoy__extensions__filters__network__http_0connection_0manager__v3__HttpConnectionManager__SetCurrentClientCertDetails_msg_init_ptr;
extern const upb_MiniTable envoy__extensions__filters__network__http_0connection_0manager__v3__HttpConnectionManager__UpgradeConfig_msg_init;
extern const upb_MiniTable* envoy__extensions__filters__network__http_0connection_0manager__v3__HttpConnectionManager__UpgradeConfig_msg_init_ptr;
extern const upb_MiniTable envoy__extensions__filters__network__http_0connection_0manager__v3__HttpConnectionManager__PathNormalizationOptions_msg_init;
extern const upb_MiniTable* envoy__extensions__filters__network__http_0connection_0manager__v3__HttpConnectionManager__PathNormalizationOptions_msg_init_ptr;
extern const upb_MiniTable envoy__extensions__filters__network__http_0connection_0manager__v3__HttpConnectionManager__ProxyStatusConfig_msg_init;
extern const upb_MiniTable* envoy__extensions__filters__network__http_0connection_0manager__v3__HttpConnectionManager__ProxyStatusConfig_msg_init_ptr;
extern const upb_MiniTable envoy__extensions__filters__network__http_0connection_0manager__v3__HttpConnectionManager__HcmAccessLogOptions_msg_init;
extern const upb_MiniTable* envoy__extensions__filters__network__http_0connection_0manager__v3__HttpConnectionManager__HcmAccessLogOptions_msg_init_ptr;
extern const upb_MiniTable envoy__extensions__filters__network__http_0connection_0manager__v3__LocalReplyConfig_msg_init;
extern const upb_MiniTable* envoy__extensions__filters__network__http_0connection_0manager__v3__LocalReplyConfig_msg_init_ptr;
extern const upb_MiniTable envoy__extensions__filters__network__http_0connection_0manager__v3__ResponseMapper_msg_init;
extern const upb_MiniTable* envoy__extensions__filters__network__http_0connection_0manager__v3__ResponseMapper_msg_init_ptr;
extern const upb_MiniTable envoy__extensions__filters__network__http_0connection_0manager__v3__Rds_msg_init;
extern const upb_MiniTable* envoy__extensions__filters__network__http_0connection_0manager__v3__Rds_msg_init_ptr;
extern const upb_MiniTable envoy__extensions__filters__network__http_0connection_0manager__v3__ScopedRouteConfigurationsList_msg_init;
extern const upb_MiniTable* envoy__extensions__filters__network__http_0connection_0manager__v3__ScopedRouteConfigurationsList_msg_init_ptr;
extern const upb_MiniTable envoy__extensions__filters__network__http_0connection_0manager__v3__ScopedRoutes_msg_init;
extern const upb_MiniTable* envoy__extensions__filters__network__http_0connection_0manager__v3__ScopedRoutes_msg_init_ptr;
extern const upb_MiniTable envoy__extensions__filters__network__http_0connection_0manager__v3__ScopedRoutes__ScopeKeyBuilder_msg_init;
extern const upb_MiniTable* envoy__extensions__filters__network__http_0connection_0manager__v3__ScopedRoutes__ScopeKeyBuilder_msg_init_ptr;
extern const upb_MiniTable envoy__extensions__filters__network__http_0connection_0manager__v3__ScopedRoutes__ScopeKeyBuilder__FragmentBuilder_msg_init;
extern const upb_MiniTable* envoy__extensions__filters__network__http_0connection_0manager__v3__ScopedRoutes__ScopeKeyBuilder__FragmentBuilder_msg_init_ptr;
extern const upb_MiniTable envoy__extensions__filters__network__http_0connection_0manager__v3__ScopedRoutes__ScopeKeyBuilder__FragmentBuilder__HeaderValueExtractor_msg_init;
extern const upb_MiniTable* envoy__extensions__filters__network__http_0connection_0manager__v3__ScopedRoutes__ScopeKeyBuilder__FragmentBuilder__HeaderValueExtractor_msg_init_ptr;
extern const upb_MiniTable envoy__extensions__filters__network__http_0connection_0manager__v3__ScopedRoutes__ScopeKeyBuilder__FragmentBuilder__HeaderValueExtractor__KvElement_msg_init;
extern const upb_MiniTable* envoy__extensions__filters__network__http_0connection_0manager__v3__ScopedRoutes__ScopeKeyBuilder__FragmentBuilder__HeaderValueExtractor__KvElement_msg_init_ptr;
extern const upb_MiniTable envoy__extensions__filters__network__http_0connection_0manager__v3__ScopedRds_msg_init;
extern const upb_MiniTable* envoy__extensions__filters__network__http_0connection_0manager__v3__ScopedRds_msg_init_ptr;
extern const upb_MiniTable envoy__extensions__filters__network__http_0connection_0manager__v3__HttpFilter_msg_init;
extern const upb_MiniTable* envoy__extensions__filters__network__http_0connection_0manager__v3__HttpFilter_msg_init_ptr;
extern const upb_MiniTable envoy__extensions__filters__network__http_0connection_0manager__v3__RequestIDExtension_msg_init;
extern const upb_MiniTable* envoy__extensions__filters__network__http_0connection_0manager__v3__RequestIDExtension_msg_init_ptr;
extern const upb_MiniTable envoy__extensions__filters__network__http_0connection_0manager__v3__EnvoyMobileHttpConnectionManager_msg_init;
extern const upb_MiniTable* envoy__extensions__filters__network__http_0connection_0manager__v3__EnvoyMobileHttpConnectionManager_msg_init_ptr;

extern const upb_MiniTableFile envoy_extensions_filters_network_http_connection_manager_v3_http_connection_manager_proto_upb_file_layout;

#ifdef __cplusplus
}  /* extern "C" */
#endif

#include "upb/port/undef.inc"

#endif  /* ENVOY_EXTENSIONS_FILTERS_NETWORK_HTTP_CONNECTION_MANAGER_V3_HTTP_CONNECTION_MANAGER_PROTO_UPB_H__UPB_MINITABLE_H_ */
