// Copyright 2023 The gRPC Authors
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
#ifndef GRPC_SRC_CORE_LIB_EVENT_ENGINE_THREAD_LOCAL_H
#define GRPC_SRC_CORE_LIB_EVENT_ENGINE_THREAD_LOCAL_H
#include <grpc/support/port_platform.h>

namespace grpc_event_engine::experimental {

/// A lightweight facility to allow gpr's fork handlers and
/// EventEngine::Forkables to coordinate.
class ThreadLocal {
 public:
  static void SetIsEventEngineThread(bool is_local);
  static bool IsEventEngineThread();
};

}  // namespace grpc_event_engine::experimental

#endif  // GRPC_SRC_CORE_LIB_EVENT_ENGINE_THREAD_LOCAL_H
