// Copyright 2024 The gRPC Authors
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

#ifndef GRPC_SRC_CORE_UTIL_GLOB_H
#define GRPC_SRC_CORE_UTIL_GLOB_H

#include "absl/strings/string_view.h"

namespace grpc_core {

// A basic glob matcher based on https://research.swtch.com/glob.
// This supports `?` (a single wildcard character), and `*` (multiple wildcard
// characters).
bool GlobMatch(absl::string_view name, absl::string_view pattern);

}  // namespace grpc_core

#endif  // GRPC_SRC_CORE_UTIL_GLOB_H
